import {
  createContext,
  Dispatch,
  PropsWithChildren,
  SetStateAction,
} from 'react';

import { DataTypeSingle } from '../types';

export type FormContextType = {
  data: object;
  updateField?: (field: string, value: unknown) => void;
  getFieldValue: (field: string) => DataTypeSingle;
  getData: () => object;
  setData?: Dispatch<SetStateAction<object>>;
  errors: Record<string, string>;
  setErrors?: Dispatch<SetStateAction<Record<string, string>>>;
  getError: (field: string) => string;
};

export const FormContext = createContext<FormContextType>({
  data: {},
  getFieldValue: () => undefined,
  getError: () => undefined,
  getData: () => ({}),
  errors: {},
});

export interface ProviderProps<DataType> extends PropsWithChildren {
  getData: () => DataType;
  getFieldValue: (field: string) => DataTypeSingle;
  updateField: (field: string, value: never) => void;
  data: Partial<DataType>;
  setData: Dispatch<SetStateAction<DataType>>;
  errors: Record<string, string>;
  setErrors: Dispatch<SetStateAction<Record<string, string>>>;
  getError: (field: string) => string;
}

const Provider = <DataType extends object>({
  children,
  getData,
  updateField,
  getFieldValue,
  data,
  setData,
    setErrors,
    errors,
    getError
}: ProviderProps<DataType>) => {
  return (
    <FormContext.Provider
      value={{ getFieldValue, getData, updateField, data, setData, errors, setErrors, getError }}
    >
      {children}
    </FormContext.Provider>
  );
};

export default Provider;
